@@ -834,6 +834,19 @@ class TestQuery(unittest.TestCase):
         assert query.encode(';') == 'a=b+c;d=e+f'
         assert query.encode(';', quote_plus=False) == 'a=b%20c;d=e%20f'
 
+        # Encode '/' consistently across quote_plus=True and quote_plus=False.
+        query = furl.Query('a /b')
+        assert query.encode(quote_plus=True) == 'a+%2Fb'
+        assert query.encode(quote_plus=False) == 'a%20%2Fb'
+
+        # dont_quote= accepts both True and a string of safe characters not to
+        # percent-encode. Unsafe query characters, like '^' and '#', are always
+        # percent-encoded.
+        query = furl.Query('a %2B/b?#')
+        assert query.encode(dont_quote='^') == 'a+%2B%2Fb%3F%23'
+        assert query.encode(quote_plus=True, dont_quote=True) == 'a++/b?%23'
+        assert query.encode(quote_plus=False, dont_quote=True) == 'a%20+/b?%23'
+
     def test_asdict(self):
         pairs = [('a', '1'), ('ロリポップ', 'testä')]
         key_encoded = '%E3%83%AD%E3%83%AA%E3%83%9D%E3%83%83%E3%83%97'
@@ -1386,6 +1399,13 @@ class TestFurl(unittest.TestCase):
         f.password = ''
         assert f.username is None and f.password == '' and f.url == '//:@'
 
+        # Unicode.
+        username = u'kødp'
+        password = u'ålæg'
+        f = furl.furl(u'https://%s:%s@example.com/' % (username, password))
+        assert f.username == username and f.password == password
+        assert f.url == 'https://k%C3%B8dp:%C3%A5l%C3%A6g@example.com/'
+
     def test_basics(self):
         url = 'hTtP://www.pumps.com/'
         f = furl.furl(url)
@@ -2026,14 +2046,27 @@ class TestFurl(unittest.TestCase):
         assert f.url == 'foo:blah'
 
     def test_tostr(self):
-        f = furl.furl('http://blast.off/?a+b=c+d&two%20tap=cat%20nap%24')
+        f = furl.furl('http://blast.off/?a+b=c+d&two%20tap=cat%20nap%24%21')
         assert f.tostr() == f.url
         assert (f.tostr(query_delimiter=';') ==
-                'http://blast.off/?a+b=c+d;two+tap=cat+nap%24')
+                'http://blast.off/?a+b=c+d;two+tap=cat+nap%24%21')
         assert (f.tostr(query_quote_plus=False) ==
-                'http://blast.off/?a%20b=c%20d&two%20tap=cat%20nap%24')
+                'http://blast.off/?a%20b=c%20d&two%20tap=cat%20nap%24%21')
         assert (f.tostr(query_delimiter=';', query_quote_plus=False) ==
-                'http://blast.off/?a%20b=c%20d;two%20tap=cat%20nap%24')
+                'http://blast.off/?a%20b=c%20d;two%20tap=cat%20nap%24%21')
+        assert (f.tostr(query_quote_plus=False, query_dont_quote=True) ==
+                'http://blast.off/?a%20b=c%20d&two%20tap=cat%20nap$!')
+        # query_dont_quote ignores invalid query characters, like '$'.
+        assert (f.tostr(query_quote_plus=False, query_dont_quote='$') ==
+                'http://blast.off/?a%20b=c%20d&two%20tap=cat%20nap$%21')
+
+        url = 'https://klugg.com/?hi=*'
+        url_encoded = 'https://klugg.com/?hi=%2A&url='
+        f = furl.furl(url).set(args=[('hi', '*'), ('url', url)])
+        assert f.tostr() == url_encoded + quote_plus(url)
+        assert f.tostr(query_dont_quote=True) == url + '&url=' + url
+        assert f.tostr(query_dont_quote='*') == (
+            url + '&url=' + quote_plus(url, '*'))
 
     def test_equality(self):
         assert furl.furl() is not furl.furl() and furl.furl() == furl.furl()
